/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is 
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Alec Flett              <alecf@netscape.com>
 *   Ben Goodger             <ben@netscape.com>
 *   Mike Pinkerton          <pinkerton@netscape.com>
 *   Blake Ross              <blakeross@telocity.com>
 *   Henk-Johan van Rantwijk <bugs4hj@netscape.net>
 *   David Perry             <d.perry@utoronto.ca>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/*
 * - [ Dependencies ] ---------------------------------------------------------
 *  utilityOverlay.js:
 *    - gatherTextUnder
 */

  var pref = null;
  pref = Components.classes["@mozilla.org/preferences-service;1"]
                   .getService(Components.interfaces.nsIPrefBranch);

  // Prefill a single text field
  function prefillTextBox(target) {
    // obtain values to be used for prefilling
    var walletService = Components.classes["@mozilla.org/wallet/wallet-service;1"].getService(Components.interfaces.nsIWalletService);
    var value = walletService.WALLET_PrefillOneElement(window._content, target);
    if (value) {

      // result is a linear sequence of values, each preceded by a separator character
      // convert linear sequence of values into an array of values
      var separator = value[0];
      var valueList = value.substring(1, value.length).split(separator);

      target.value = valueList[0];
    }
  }
  
  // Called whenever the user clicks in the content area,
  // except when left-clicking on links (special case)
  // should always return true for click to go through
  function contentAreaClick(event) 
  {
    var target = event.target;
    var linkNode;

    var local_name = target.localName;

    if (local_name) {
      local_name = local_name.toLowerCase();
    }

    switch (local_name) {
      case "a":
      case "area":
      case "link":
        if (target.hasAttribute("href")) 
          linkNode = target;
        break;
      case "input":
        if (pref.getBoolPref("multizilla.forced.autocomplete")) {
          forcedAutocomplete(event);
        }
        if ((event.target.type.toLowerCase() == "text" || event.target.type == "") // text field
            && event.detail == 2 // double click
            && event.button == 0 // left mouse button
            && event.target.value.length == 0) { // no text has been entered
          prefillTextBox(target); // prefill the empty text field if possible
        }
        if (event.target.type.toLowerCase() == "submit") {
          if (pref.getBoolPref("multizilla.forced.autocomplete")) {
            forcedAutocomplete(event);
          }
          if (event.target.parentNode.hasAttribute("target") && pref.getBoolPref("multizilla.forced.form-targets")) { // this should intercepts form submits
            var targetID = event.target.parentNode.target.toLowerCase();
            // if (targetID == "_new" || targetID == "new" || targetID == "_blank" || targetID == "blank") {
            if (targetID != "self" && targetID != "top") {
              event.target.parentNode.removeAttribute("target"); // load in current page
              return true;
            }
          }
        }
        break;
      case "password":
        if (pref.getBoolPref("multizilla.forced.autocomplete")) {
          forcedAutocomplete(event);
        }
        break;
      default:
        linkNode = findParentNode(event.originalTarget, "a");
        // <a> cannot be nested.  So if we find an anchor without an
        // href, there is no useful <a> around the target
        if (linkNode && !linkNode.hasAttribute("href"))
          linkNode = null;
        break;
    }
    if (linkNode) {
      if (handleLinkClick(event, linkNode.href, linkNode) == -1)
        return false;
      else return true;
    } 
    else {
      // Try simple XLink
      var href;
      linkNode = target;
      while (linkNode) {
        if (linkNode.nodeType == Node.ELEMENT_NODE) {
          href = linkNode.getAttributeNS("http://www.w3.org/1999/xlink", "href");
          break;
        }
        linkNode = linkNode.parentNode;
      }
      if (href && href != "") {
        href = makeURLAbsolute(target.baseURI,href);
        handleLinkClick(event, href, null);
        return true;
      }
    }
    if (pref && event.button == 1 && !findParentNode(event.originalTarget, "scrollbar") &&
        pref.getBoolPref("middlemouse.contentLoadURL")) {
      if (middleMousePaste())
        event.preventBubble();
    }
    return true;
  }

  function handleLinkClick(event, href, linkNode)
  {
    // Make sure we are allowed to open this URL
    urlSecurityCheck(href, document);
    var sourceTab = getBrowser().mCurrentTab;

    switch (event.button) {
      case 0:                                                         // if left button clicked
        if (event.metaKey || event.ctrlKey) {                         // and meta or ctrl are down
          if (openLinkInNewWindow(event, href) || openLinkInNewTab(event, href)) {
            // setLinkVisited(linkNode, href);
            return true;
          }
        }
        var saveModifier = true;
        if (pref) {
          try {
            saveModifier = pref.getBoolPref("ui.key.saveLink.shift");
          }
          catch(ex) {            
          }
        }
        saveModifier = saveModifier ? event.shiftKey : event.altKey;
          
        if (saveModifier) {                                           // if saveModifier is down
          saveURL(href, linkNode ? gatherTextUnder(linkNode) : "");
          return true;
        }
        if (event.altKey)                                             // if alt is down
          return true;                                                // do nothing
        if (pref && pref.getBoolPref("multizilla.newtabfor.windowopen") && 
            (linkNode.hasAttribute("target") || hasTarget("base", event))) {
          var targetID = null;
          if (linkNode.hasAttribute("target")) {
            targetID = linkNode.target;
          }
          else {
            var base = event.target.ownerDocument.getElementsByTagName("base");
            targetID = base[0].hasAttribute("target") ? base[0].getAttribute("target") : null;
          }
          if (targetID == "_content") { // see mz bugzilla bug 1706
            return true;
          }
          if (preventDupeTab(href)) {
            event.preventBubble();
            return -1;
          }
          if (_content.frames.length || isContentFrame(document.commandDispatcher.focusedWindow)) {
            for (var frame_nr = 0; frame_nr < _content.frames.length; frame_nr++) {
              if (_content.frames[frame_nr].length) { // deep nested
                if (hasChildFrameTargetID(_content.frames[frame_nr], targetID))
                  return true;
              }
              if (targetID == _content.frames[frame_nr].name)
                return true;
            }
          }
          if (hasTargetNew(targetID) && targetID != "") { // do we need to open a new tab?
            openLinkInNewTab(event, href, targetID); // see mz bugzilla bug 1832
            // setLinkVisited(linkNode, href);
            return -1;
          }
          else {
            return true;
          }
        }
        if (readMyPref("multizilla.refspoof.spoofing", "int", 1) != 1) {
          if (linkNode.ownerDocument != window._content.document || linkNode.getAttribute("href") == "#" || 
              linkNode.hasAttribute("target") || hasTarget("base", event))
            return true;
          else { 
            event.preventDefault();
            var referrer = getReferrer(document);
            getBrowser().loadURI(href, nsIWebNavigation.LOAD_FLAGS_NONE, referrer, null, null);
            return -1;
          }
        }
		return false;
      case 1: // middle button clicked
        if (href.toLowerCase().search( "mailto:" ) == 0) {
          loadURI(href);
          return false;
        }
        if (openLinkInNewTab(event, href) || openLinkInNewWindow(event, href)) {
          // setLinkVisited(linkNode, href);
          return true;
        }
        break;
    }
    return false;
  }

function middleMousePaste()
{
  var url = readFromClipboard();
  if (url) {
    loadURI(getShortcutOrURI(url));
    return true;
  }
  return false;
}

function makeURLAbsolute(base, url) 
{
  // Construct nsIURL.
  var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  var baseURI  = ioService.newURI(base, null, null);
  return ioService.newURI(baseURI.resolve(url), null, null).spec;
}

function hasTarget(name, event)
{
  var sourceDoc = event.target.ownerDocument;
  var node = sourceDoc.getElementsByTagName(name);

  for (var i = 0; i < node.length; ++i) {
    if (node[i].hasAttribute("target"))
      return true;
  }
  return false;
}

function hasTargetNew(target)
{
  target = target.toLowerCase();
  // only the first three are conform HTML specifications
  if (target != "_self" && target != "_top" && target != "_parent" && 
      target != "self" && target != "top" && target != "parent") {
    return true;
  }
  return false;
}

function forcedAutocomplete(event)
{
  var sourceDoc = event.target.ownerDocument;
  var node = event.target;

  if (node.hasAttribute("autocomplete")) {
    if (node.getAttribute("autocomplete").toLowerCase() == "off") {
      node.setAttribute("autocomplete", "on");
      // return;
    }
  }  
  node = sourceDoc.getElementsByTagName("form");
  var onsubmitRemoval = pref.getBoolPref("multizilla.forced.onsubmit-remove");

  for (var i = 0; i < node.length; ++i) {
    if (onsubmitRemoval) {
      if (node[i].hasAttribute("onsubmit"))
        node[i].removeAttribute("onsubmit");
    }
    if (node[i].hasAttribute("autocomplete")) {
      if (node[i].getAttribute("autocomplete").toLowerCase() == "off") {
        node[i].setAttribute("autocomplete", "on");
      }
    }
  }
  return;
}

function openLinkInNewTab(event, href, target)
{
  var chromeHidden = document.documentElement.hasAttribute("chromehidden");
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var enumerator = windowManagerInterface.getEnumerator("navigator:browser");
  var firstWindow = enumerator.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);

  if (pref && pref.getBoolPref("browser.tabs.opentabfor.middleclick") &&
      ("getBrowser" in window) && getBrowser().localName == "tabbrowser") {
    if (preventDupeTab(href))
      return true;
    // Chromeless window only, prevent hidden tabs (see also bug 2540)
    if (firstWindow.document.documentElement.hasAttribute("chromehidden"))
      document.documentElement.removeAttribute("chromehidden");
    var loadInBackground = pref.getBoolPref("browser.tabs.loadInBackground");
    var aTab = checkForBlankTab();
    var aBrowser = getBrowser(); // set default browser
    if (chromeHidden) // 
      aBrowser = firstWindow.gBrowser;
    if (aTab)
      aTab.loadURI(href, getReferrer(document));
    else aTab = aBrowser.addTab(href, getReferrer(document), target);
    if (event.shiftKey)
      loadInBackground = !loadInBackground;
    if (!loadInBackground || chromeHidden)
      aBrowser.selectedTab = aTab;
    var linkNode = findParentNode(event.originalTarget, "a");
    setLinkVisited(linkNode, href);
    event.preventBubble();
  }
  else {
    openNewWindowWith(href);
    event.preventBubble();
  }
  return true;
}

function openLinkInNewWindow(event, href)
{
  if (pref && pref.getBoolPref("middlemouse.openNewWindow")) {  
    openNewWindowWith(href);
    var linkNode = findParentNode(event.originalTarget, "a");
    setLinkVisited(linkNode, href);
    event.preventBubble();
    return true;
  }
  return false;
}

function hasChildFrameTargetID(aFrame, frameName)
{
  if (!aFrame || !frameName)
    return false;
  for (var frame_nr = 0; frame_nr < aFrame.length; frame_nr++) {
    if (aFrame.frames[frame_nr].length) {
      if (hasChildFrameTargetID(aFrame.frames[frame_nr], frameName))
        return true;
    }
    if (aFrame.frames[frame_nr].name == frameName)
      return true;
  }
  return false;
}

function setLinkVisited(aNode, aURI)
{
  if (!aNode)
    return;
  var globalHistory = Components.classes["@mozilla.org/browser/global-history;1"]
                                .getService(Components.interfaces.nsIGlobalHistory);
  try {
    if (!aURI.match(/^(https?|chrome|file|ftp|resource|gopher):/) || globalHistory.isVisited(aURI)) 
      return;
    var currentHref = aNode.href;
    globalHistory.addPage(aURI);
    aNode.href = "";
    aNode.href = currentHref;
  }
  catch(ex) {
  }
}
